<?php
/*
################################
#POSTGRESTATS ENTERPRISE LICENSE
################################
Copyright 2013 NWLYNX <http://www.nwlynx.com/>.

This file is part of PostgreStats Enterprise <http://www.postgrestats.com/>.

PostgreStats Enterprise is free software: you can redistribute it and/or modify it under the terms of the GNU General Public 
License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

PostgreStats Enterprise is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
*/

########################################################################
#SCRIPT ERROR REPORTING - UNCOMMENT THIS TO VIEW SCRIPT EXECUTION ERRORS
########################################################################
error_reporting(E_ALL);

####################
#REQUIRE PS_CONF.PHP
####################
require("ps_conf.php");

#######################################
#FUNCTION TO EXIT WITH COMMAND LINE MSG
#######################################
function do_exit($msg)
{
	############
	#ECHO & DONE
	############
	echo $msg;
	exit;
}

#########################################
#VERIFY POSTGRESTATS DATABASE CREDENTIALS
#########################################
if(empty($postgrestats_db_name) || empty($postgrestats_db_user) || empty($postgrestats_db_host) || empty($postgrestats_db_port))
{
	do_exit("You must first configure ps_conf.php (in the scripts directory of your install) with your PostgreStats database credentials.\n");
}

###################################################################
#CONNECT ATTEMPT - PASSWORD MAY NOT BE REQUIRED CONNECT ACCORDINGLY
###################################################################
if(!empty($postgrestats_db_pass))
{
	########################
	#ATTEMPT CONNECT WITH PW
	########################
	$pg_conn = @pg_connect("dbname=$postgrestats_db_name user=$postgrestats_db_user password=$postgrestats_db_pass host=$postgrestats_db_host port=$postgrestats_db_port");
}
else
{
	###########################
	#ATTEMPT CONNECT WITHOUT PW
	###########################
	$pg_conn = @pg_connect("dbname=$postgrestats_db_name user=$postgrestats_db_user host=$postgrestats_db_host port=$postgrestats_db_port");
}

########################
#DOUBLE CHECK CONNECTION
########################
if(!$pg_conn)
{
	do_exit("There was an error connecting to your PostgreStats database. Please verify the connect credentials in your ps_conf.php file located in the scripts directory of your PostgreStats install.\n");
}

###########################################################
#QUERY SERVER LIST - GRAB ALL THAT ARE NOT IN PAUSED STATUS
###########################################################
$sqlselect_server = @pg_query($pg_conn, "SELECT * FROM ps_server_tbl WHERE pause = 1");

#############
#MAKE CERTAIN
#############
if(!$sqlselect_server)
{
	do_exit("Error executing PostgreStats query.\n");
}

#####################
#CHECK FOR NO SERVERS
#####################
if(pg_num_rows($sqlselect_server) == 0)
{
	do_exit("There are no eligible servers in your PostgreStats configuration.\n");
}

###########################
#START OUR SERVER ITERATION
###########################
while($query_data_server = pg_fetch_row($sqlselect_server))
{
	#############
	#GRAB RESULTS
	#############
	$server_id = $query_data_server[0];
	$server_name = $query_data_server[1];
	$server_host = $query_data_server[2];
	$port = $query_data_server[3];

	################################################
	#QUERY DBS ON THIS SERVER - WHICH ARE NOT PAUSED
	################################################
	$sqlselect_dbs = @pg_query($pg_conn, "SELECT * FROM ps_db_tbl WHERE server_id = $server_id AND pause = 1 ORDER BY db_name ASC");

	#############
	#MAKE CERTAIN
	#############
	if(!$sqlselect_dbs)
	{
		do_exit("Error executing PostgreStats query.\n");
	}

	##################
	#CHECK FOR RESULTS
	##################
	if(pg_num_rows($sqlselect_dbs) > 0)
	{	
		##############################################
		#START OUR DB ITERATION - FIRST GRAB DB VALUES
		##############################################
		while($query_data_dbs = pg_fetch_row($sqlselect_dbs))
		{
			$db_id = $query_data_dbs[0];
			$db_last_reset = $query_data_dbs[2];
			$db_name = $query_data_dbs[3];
			$db_auth_1 = $query_data_dbs[4];
			$db_auth_2 = $query_data_dbs[5];
			$db_auth_3 = $query_data_dbs[6];
			$db_auth_4 = $query_data_dbs[7];

			########################################
			#HANDLE AUTH 1/2 - DECODE, SPLIT & STRIP
			########################################
			if(!empty($db_auth_1) && !empty($db_auth_2))
			{
				$db_auth_2 = base64_decode($db_auth_2);
				$db_auth_remove = str_split($db_auth_1);
				$db_user = strrev(str_replace($db_auth_remove,'',$db_auth_2));
				$db_user = pg_escape_string($db_user);
			}

			########################################
			#HANDLE AUTH 3/4 - DECODE, SPLIT & STRIP
			########################################
			if(!empty($db_auth_3) && !empty($db_auth_4))
			{
				$db_auth_4 = base64_decode($db_auth_4);
				$db_auth_remove = str_split($db_auth_3);
				$db_pass = strrev(str_replace($db_auth_remove,'',$db_auth_4));
				$db_pass = pg_escape_string($db_pass);
			}

			#################
			#PREP VARS FOR DB
			#################
			$db_name = pg_escape_string($db_name);

			##########################################
			#ATTEMPT TO OPEN AN INDIPENDENT CONNECTION
			##########################################
			$pg_conn_indi_con = @pg_connect("dbname=$db_name user=$db_user password=$db_pass host=$server_host port=$port",PGSQL_CONNECT_FORCE_NEW);

			########################
			#IF WE HAVE A CONNECTION
			########################
			if($pg_conn_indi_con)
			{
				################################################
				#FIRST QUERY PRIMARY STATS FROM PG_STAT_DATABASE
				################################################
				$sqlselect_stats_primary = @pg_query($pg_conn_indi_con, "SELECT datid,datname,xact_commit,xact_rollback,blks_read,blks_hit,stats_reset FROM pg_stat_database WHERE datname = '$db_name'");

				################
				#INITIALIZE VARS
				################
				$db_datid = '';
				$db_datname = '';
				$db_commits = 0;
				$db_rollbacks = 0;
				$db_block_reads = 0;
				$db_buffer_hits = 0;
				$db_reset = '';

				#############################
				#MAKE CERTAIN QUERY SUCCEEDED
				#############################
				if($sqlselect_stats_primary)
				{
					###########################
					#MAKE SURE WE HAVE A RESULT
					###########################
					if(pg_num_rows($sqlselect_stats_primary) > 0)
					{
						#########
						#FETCH IT
						#########
						$query_data_primary = pg_fetch_row($sqlselect_stats_primary);

						#########################
						#CREATE PRIMARY STAT VARS
						#########################
						$db_datid = $query_data_primary[0];
						$db_datname = $query_data_primary[1];
						$db_commits = $query_data_primary[2];
						$db_rollbacks = $query_data_primary[3];
						$db_block_reads = $query_data_primary[4];
						$db_buffer_hits = $query_data_primary[5];
						$db_reset = $query_data_primary[6];
					}
				}

				##########################
				#NOW QUERY USER STATISTICS
				##########################
				$sqlselect_stats_user = @pg_query($pg_conn_indi_con, "SELECT SUM(n_tup_ins),SUM(n_tup_upd),SUM(n_tup_del),SUM(seq_scan),SUM(idx_scan),SUM(seq_tup_read),SUM(idx_tup_fetch) FROM pg_stat_user_tables");

				################
				#INITIALIZE VARS
				################
				$db_user_inserts = 0;
				$db_user_updates = 0;
				$db_user_deletes = 0;
				$db_user_seq_scans = 0;
				$db_user_idx_scans = 0;
				$db_user_seq_rows = 0;
				$db_user_idx_rows = 0;

				#############################
				#MAKE CERTAIN QUERY SUCCEEDED
				#############################
				if($sqlselect_stats_user)
				{
					###########################
					#MAKE SURE WE HAVE A RESULT
					###########################
					if(pg_num_rows($sqlselect_stats_user) > 0)
					{
						#########
						#FETCH IT
						#########
						$query_data_user = pg_fetch_row($sqlselect_stats_user);

						######################
						#CREATE USER STAT VARS
						######################
						$db_user_inserts = $query_data_user[0];
						$db_user_updates = $query_data_user[1];
						$db_user_deletes = $query_data_user[2];
						$db_user_seq_scans = $query_data_user[3];
						$db_user_idx_scans = $query_data_user[4];
						$db_user_seq_rows = $query_data_user[5];
						$db_user_idx_rows = $query_data_user[6];
					}
				}

				##########
				#PREP VARS
				##########
				$db_datname = pg_escape_string($db_datname);

				#################################
				#CONVERT DB RESET (FROM EXTERNAL)
				#################################
				$db_last_reset_indi = strtotime($db_reset);

				##########################################
				#CONVERT DB LAST RESET (FROM POSTGRESTATS)
				##########################################
				$db_last_reset_pg = strtotime($db_last_reset);

				###############
				#COMPARE RESETS
				###############
				if($db_last_reset_indi <= $db_last_reset_pg)
				{
					######################################################
					#STATS HAVE NOT BEEN RESET - GET SUMS SINCE LAST RESET
					######################################################
					$stats_query  = "SELECT SUM(core_commits), SUM(core_rollbacks), SUM(core_block_reads), SUM(core_buffer_hits), SUM(user_inserts), SUM(user_updates), SUM(user_deletes), SUM(user_seq_scans), SUM(user_idx_scans), SUM(user_seq_rows), SUM(user_idx_rows) ";
					$stats_query .= "FROM ps_core_tbl WHERE db_id = $db_id AND recorded > '$db_reset'";
					$sqlselect_stats_dif = @pg_query($pg_conn, $stats_query);

					#############
					#MAKE CERTAIN
					#############
					if($sqlselect_stats_dif)
					{
						################
						#IF WE HAVE DATA
						################
						if(pg_num_rows($sqlselect_stats_dif) > 0)
						{
							###################
							#FETCH OUR DIF DATA
							###################
							$query_data_stats_dif = pg_fetch_row($sqlselect_stats_dif);

							#########################################
							#SUBTRACT DIF SUMS FROM CURRENT STAT SUMS
							#########################################
							$db_commits = $db_commits - $query_data_stats_dif[0];
							$db_rollbacks = $db_rollbacks - $query_data_stats_dif[1];
							$db_block_reads = $db_block_reads - $query_data_stats_dif[2];
							$db_buffer_hits = $db_buffer_hits - $query_data_stats_dif[3];

							###########
							#USER STATS
							###########
							$db_user_inserts = $db_user_inserts - $query_data_stats_dif[4];
							$db_user_updates = $db_user_updates - $query_data_stats_dif[5];
							$db_user_deletes = $db_user_deletes - $query_data_stats_dif[6];
							$db_user_seq_scans = $db_user_seq_scans - $query_data_stats_dif[7];
							$db_user_idx_scans = $db_user_idx_scans - $query_data_stats_dif[8];
							$db_user_seq_rows = $db_user_seq_rows - $query_data_stats_dif[9];
							$db_user_idx_rows = $db_user_idx_rows - $query_data_stats_dif[10];
						}
					}
				}

				###################################################
				#FIND OUT IF STATS HAVE ALREADY BEEN RECORDED TODAY
				###################################################
				$sqlselect_ps_core_id = @pg_query($pg_conn, "SELECT ps_core_id FROM ps_core_tbl WHERE db_id = $db_id AND DATE(recorded) = CURRENT_DATE");

				#############
				#MAKE CERTAIN
				#############
				if($sqlselect_ps_core_id)
				{
					#####################################################
					#IF NO STATS FOR TODAY - PREP QUERY FOR STRAIGHT DUMP
					#####################################################
					if(pg_num_rows($sqlselect_ps_core_id) == 0)
					{
						#############
						#QUERY STRING
						#############
						$stats_query  = "INSERT INTO ps_core_tbl(db_id, server_id, recorded, datid, datname, core_commits, core_rollbacks, core_block_reads, core_buffer_hits, user_inserts, user_updates, user_deletes, user_seq_scans, user_idx_scans, user_seq_rows, user_idx_rows) ";
						$stats_query .= "VALUES($db_id, $server_id, CURRENT_TIMESTAMP, $db_datid, '$db_datname', $db_commits, $db_rollbacks, $db_block_reads, $db_buffer_hits, $db_user_inserts, $db_user_updates, $db_user_deletes, $db_user_seq_scans, $db_user_idx_scans, $db_user_seq_rows, $db_user_idx_rows)";
					}
					else
					{
						###################################
						#GET DIFFERENCE QUERY FOR THIS DUMP
						###################################
						$query_data_ps_core_id = pg_fetch_row($sqlselect_ps_core_id);
						$ps_core_id = $query_data_ps_core_id[0];

						################################################
						#UPDATE TODAY'S RECORD - APPLYING NEW STATISTICS
						################################################
						$stats_query  = "UPDATE ps_core_tbl SET ";
						$stats_query .= "recorded = CURRENT_TIMESTAMP, ";
						$stats_query .= "core_commits = core_commits + $db_commits, ";
						$stats_query .= "core_rollbacks = core_rollbacks + $db_rollbacks, ";
						$stats_query .= "core_block_reads = core_block_reads + $db_block_reads, ";
						$stats_query .= "core_buffer_hits = core_buffer_hits + $db_buffer_hits, ";
						$stats_query .= "user_inserts = user_inserts + $db_user_inserts, ";
						$stats_query .= "user_updates = user_updates + $db_user_updates, ";
						$stats_query .= "user_deletes = user_deletes + $db_user_deletes, ";
						$stats_query .= "user_seq_scans = user_seq_scans + $db_user_seq_scans, ";
						$stats_query .= "user_idx_scans = user_idx_scans + $db_user_idx_scans, ";
						$stats_query .= "user_seq_rows = user_seq_rows + $db_user_seq_rows, ";
						$stats_query .= "user_idx_rows = user_idx_rows + $db_user_idx_rows ";
						$stats_query .= "WHERE ps_core_id = $ps_core_id";
					}
				}

				#############################################
				#PERFORM QUERY ON THE POSTGRESTATS CONNECTION
				#############################################
				$sqlinsert_stats = @pg_query($pg_conn, $stats_query);

				#####################################################################
				#UPDATE PS_DB_TBL WITH LAST RECORDED STAT RESET FROM PG_STAT_DATABASE
				#####################################################################
				$sqlupdate_postgrestats_reset = @pg_query($pg_conn, "UPDATE ps_db_tbl SET db_last_reset = (SELECT stats_reset FROM pg_stat_database WHERE datname = '$db_name') WHERE db_id = $db_id");

				##########################
				#CLOSE THE INDI CONNECTION
				##########################
				pg_close($pg_conn_indi_con);
			}
		}
	}
}

#####################
#DONE RECORDING STATS
#####################
echo "PostgreStats Recorded.\n";
exit;

?>